
#include "SpikeRenderers.h"

#include "../util/Random.h"


void CLineRenderer::renderSpike( const CSpike& spike, const SVector& v1, const SVector& v2, float scale )
{
	int x1 = (int)(v1.x) + mColor->getWidth()/2;
	int y1 = -(int)(v1.y) + mColor->getHeight()/2;
	int x2 = (int)(v2.x) + mColor->getWidth()/2;
	int y2 = -(int)(v2.y) + mColor->getHeight()/2;

	unsigned int c = (unsigned int)( (v1.z+v2.z)*0.5f * mFogMult );
	unsigned int r, g, b;
	if( !mDark ) {
		r = 96 + c/2;
		g = 96 + c/2;
		b = 160 + c/4;
	} else {
		r = 192 - c/2;
		g = 192 - c/2;
		b = 255 - c/2;
	}

	float len = spike.getDirection().length() * scale;
	int pof = (int)(len / 64);
	int alp = ((int)len) * 128 / mColor->getHeight();
	if( alp > 24 )
		alp = 24;
	
	mDrawing->lineBlendZ( *mColor, x1+pof, y1, x2, y2, CCanvas::rgb(r,g,b), alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1-pof, y1, x2, y2, CCanvas::rgb(r,g,b), alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1, y1+pof, x2, y2, CCanvas::rgb(r,g,b), alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1, y1-pof, x2, y2, CCanvas::rgb(r,g,b), alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1, y1, x2, y2, CCanvas::rgb(r,g,b), alp + 7, *mDepth, v1.z, v2.z );
}


CRandom CFuzzyLineRenderer::mRandom;

void CFuzzyLineRenderer::renderSpike( const CSpike& spike, const SVector& v1, const SVector& v2, float scale )
{
	int x1 = (int)(v1.x) + mColor->getWidth()/2;
	int y1 = -(int)(v1.y) + mColor->getHeight()/2;
	int x2 = (int)(v2.x) + mColor->getWidth()/2;
	int y2 = -(int)(v2.y) + mColor->getHeight()/2;

	unsigned int c = (unsigned int)( (v1.z+v2.z)*0.5f * mFogMult );
	unsigned int r, g, b;
	if( !mDark ) {
		r = 96 + c/2;
		g = 96 + c/2;
		b = 160 + c/4;
	} else {
		r = 192 - c/2;
		g = 192 - c/2;
		b = 220 - c/2;
	}
	
	CCanvas::pixel color1 = CCanvas::rgb( r+32, g+32, b+32 );
	CCanvas::pixel color2 = CCanvas::rgb( r+16, g+16, b+32 );
	CCanvas::pixel color3 = CCanvas::rgb( r, g, b+32 );
	CCanvas::pixel color4 = CCanvas::rgb( r, g, b );
	CCanvas::pixel colorMain = CCanvas::rgb( r, g, b );

	int alp = mDark ? 8 : 12;
	int alpMain = mDark ? 12 : 16;
	
	mDrawing->lineBlendZ( *mColor, x1+(mRandom.getInt()&7), y1-(mRandom.getInt()&7), x2-(mRandom.getInt()&7), y2+(mRandom.getInt()&7), color1, alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1-(mRandom.getInt()&7), y1-(mRandom.getInt()&7), x2+(mRandom.getInt()&7), y2-(mRandom.getInt()&7), color2, alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1+(mRandom.getInt()&7), y1+(mRandom.getInt()&7), x2-(mRandom.getInt()&7), y2-(mRandom.getInt()&7), color3, alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1-(mRandom.getInt()&7), y1+(mRandom.getInt()&7), x2+(mRandom.getInt()&7), y2+(mRandom.getInt()&7), color4, alp, *mDepth, v1.z, v2.z );
	mDrawing->lineBlendZ( *mColor, x1, y1, x2, y2, colorMain, alpMain, *mDepth, v1.z, v2.z );
}


