
#include "SnowFlakeBuilder.h"


#define PI 3.1415927f


void CSnowFlakeBuilder::buildRules( CSpikeSystem& s )
{
	CSpikeRule *fr;
	
	// category 0
	fr = new CSpikeRule( 1.0f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 0.3f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/16, PI/16, PI/16, 0.7f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/4, PI/4, PI/4, 0.6f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/4, PI/4, PI/4, 0.5f, 0.3f, true ) );
	s.getRules(0).push_back( fr );
	
	// category 1
	fr = new CSpikeRule( 1.0f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/4, PI/4, PI/4, 0.5f, 0.4f, true ) );
	s.getRules(1).push_back( fr );
}

void CSnowFlakeBuilder::buildInitial( CSpikeSystem& s )
{
	// prepopulate
	for( int i = 0; i < mInitialSpikes; i++ ) {
		float z = (s.getRandom().getFloat() - 0.5f) * 2.0f;
		float q = s.getRandom().getFloat() * 2.0f * PI;
		float r = sqrtf( 1.0f - z*z );
		float x = r * cosf( q );
		float y = r * sinf( q );
		s.getSystem().addSpike( 0, CSpike( SVector(0,0,0), SVector(x,y,z) ) );
	}
}


void CSnowFlakeBuilder2::buildRules( CSpikeSystem& s )
{
	CSpikeRule *fr;
	
	// category 0
	fr = new CSpikeRule( 0.8f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 0.3f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/8, PI/8, PI/8, 0.7f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.7f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.6f, 0.3f, true ) );
	s.getRules(0).push_back( fr );
	fr = new CSpikeRule( 0.2f );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, 0.0f, 0.3f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/2, PI/2, PI/2, 0.3f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/2, PI/2, PI/2, 0.3f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/2, PI/2, PI/2, 0.3f, 0.1f, true ) );
	s.getRules(0).push_back( fr );
	
	// category 1
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.9f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.8f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.8f, 0.3f, true ) );
	s.getRules(1).push_back( fr );
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 2, PI/3, PI/3, PI/3, 0.9f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/3, PI/3, PI/3, 0.8f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 2, PI/3, PI/3, PI/3, 0.8f, 0.3f, true ) );
	s.getRules(1).push_back( fr );
	
	// category 2
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 2, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/4, PI/4, PI/4, 0.8f, 0.1f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/3, PI/3, PI/3, 0.8f, 0.3f, true ) );
	s.getRules(2).push_back( fr );
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/4, PI/4, PI/4, 0.8f, 0.1f, true ) );
	s.getRules(2).push_back( fr );
}



void CSnowFlakeBuilder3::buildRules( CSpikeSystem& s )
{
	CSpikeRule *fr;
	
	// category 0
	fr = new CSpikeRule( 0.8f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 0.3f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/30, PI/30, PI/8, 0.7f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/30, PI/30, PI/3, 0.7f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/30, PI/30, PI/3, 0.6f, 0.3f, true ) );
	s.getRules(0).push_back( fr );
	fr = new CSpikeRule( 0.2f );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, 0.0f, 0.3f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/6, PI/30, PI/2, 0.3f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/10, PI/30, PI/8, 0.3f, 0.2f, false ) );
	fr->addModifier( CSpikeModifier( 0, PI/10, PI/30, PI/2, 0.3f, 0.1f, true ) );
	s.getRules(0).push_back( fr );
	
	// category 1
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/30, PI/3, PI/9, 0.9f, 0.2f, false ) );
	fr->addModifier( CSpikeModifier( 0, PI/20, PI/7, PI/12, 0.8f, 0.3f, false ) );
	fr->addModifier( CSpikeModifier( 0, PI/20, PI/8, PI/10, 0.8f, 0.3f, false ) );
	s.getRules(1).push_back( fr );
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 2, PI/10, PI/3, PI/3, 0.9f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/6, PI/3, PI/3, 0.8f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 2, PI/30, PI/3, PI/3, 0.8f, 0.3f, true ) );
	s.getRules(1).push_back( fr );
	
	// category 2
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 2, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/10, PI/4, PI/4, 0.8f, 0.1f, true ) );
	fr->addModifier( CSpikeModifier( 0, PI/10, PI/3, PI/3, 0.8f, 0.3f, true ) );
	s.getRules(2).push_back( fr );
	fr = new CSpikeRule( 0.5f );
	fr->addModifier( CSpikeModifier( 1, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, PI/10, PI/4, PI/4, 0.8f, 0.1f, true ) );
	s.getRules(2).push_back( fr );
}


void CSnowFlakeBuilder4::buildRules( CSpikeSystem& s )
{
	CSpikeRule *fr;
	
	// category 0
	fr = new CSpikeRule( 0.3f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.3f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.3f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.6f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.6f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.2f, 0.1f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.2f, 0.1f ) );
	s.getRules(0).push_back( fr );
	fr = new CSpikeRule( 0.3f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.35f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4.6f, 0.35f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.8f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/3.6f, 0.7f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.3f, 0.1f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.3f, 0.1f ) );
	s.getRules(0).push_back( fr );
	fr = new CSpikeRule( 0.3f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.35f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.35f, 0.7f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/4, 0.75f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, -PI/4, 0.7f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, PI/4, 0.3f, 0.1f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, -PI/3.5f, 0.3f, 0.1f ) );
	s.getRules(0).push_back( fr );
	fr = new CSpikeRule( 0.1f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, PI/4.5f, 0.2f, 0.6f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, -PI/3, 0.3f, 0.6f ) );
	fr->addModifier( CSpikeModifier( 0, 0.0f, 0.0f, PI/3, 0.7f, 0.3f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, PI/2, 0.25f, 0.1f ) );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, -PI/2, 0.2f, 0.1f ) );
	s.getRules(0).push_back( fr );
	
	// category 3
	fr = new CSpikeRule( 1.0f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	s.getRules(3).push_back( fr );
}


void CSnowFlakeBuilder4::buildInitial( CSpikeSystem& s )
{
	// prepopulate
	for( int i = 0; i < 6; i++ ) {
		float x = cosf((float)i*(PI/3));
		float y = sinf((float)i*(PI/3));
		float z = 0.0f;
		s.getSystem().addSpike( 0, CSpike( SVector(0,0,0), SVector(x,y,z) ) );
	}
}



void CXmasTreeBuilder::buildRules( CSpikeSystem& s )
{
	CSpikeRule *fr;
	
	// category 0
	fr = new CSpikeRule( 1.0f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI*2/3, 0.1f+PI*2/5, 0.0f, 1.0f, 1.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI*2/3, 0.1f+PI*2/5*2, 0.0f, 1.0f, 1.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI*2/3, 0.1f+PI*2/5*3, 0.0f, 1.0f, 1.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI*2/3, 0.1f+PI*2/5*4, 0.0f, 1.0f, 1.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI*2/3, 0.1f+PI*2/5*5, 0.0f, 1.0f, 1.0f ) );
	s.getRules(0).push_back( fr );
	
	// category 1
	fr = new CSpikeRule( 0.7f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/6, PI/6, PI/6, 0.6f, 0.4f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/6, PI/6, PI/6, 0.6f, 0.4f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.7f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.7f, true ) );
	s.getRules(1).push_back( fr );
	fr = new CSpikeRule( 0.3f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.5f, 0.1f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.2f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/6, PI/6, PI/6, 0.6f, 0.4f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/6, PI/6, PI/6, 0.6f, 0.3f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.7f, true ) );
	fr->addModifier( CSpikeModifier( 1, PI/5, PI/5, PI/5, 0.4f, 0.7f, true ) );
	s.getRules(1).push_back( fr );
	

	// category 3
	fr = new CSpikeRule( 1.0f );
	fr->addModifier( CSpikeModifier( 3, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f ) );
	s.getRules(3).push_back( fr );
}


void CXmasTreeBuilder::buildInitial( CSpikeSystem& s )
{
	// prepopulate
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,0.00f,0.0f), SVector(0,0.64f,0.00f) ) );
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,0.64f,0.0f), SVector(0,0.32f,0.00f) ) );
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,0.96f,0.0f), SVector(0,0.16f,0.00f) ) );
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,1.12f,0.0f), SVector(0,0.08f,0.00f) ) );
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,1.20f,0.0f), SVector(0,0.04f,0.00f) ) );
	s.getSystem().addSpike( 0, CSpike( SVector(0.0f,1.24f,0.0f), SVector(0,0.02f,0.00f) ) );
}
